/*
------------------------------------------------------------
	window.h - Generic Rendering Window
------------------------------------------------------------
 *  David Olofson, 2001
 * This code is released under the terms of the GPL.
 */

#include "window.h"
#include "gfxengine.h"
#include "sofont.h"

#define	SELECT	if(selected != this) select();

window_t *window_t::selected = NULL;

window_t::window_t()
{
	engine = NULL;
	rect.x = rect.y = 0;
	rect.w = 320;
	rect.h = 240;
	vx = vy = 0;
	gvx = gvy = 0;
	__gvx = __gvy = 0;
	vw = vh = 512;
	fgcolor = bgcolor = 0;
	selected = 0;
}


window_t::~window_t()
{
	if(selected == this)
		selected = NULL;
}


void window_t::init(gfxengine_t *e)
{
	engine = e;
}


void window_t::place(int left, int top,
		int sizex, int sizey,
		int vsizex, int vsizey)
{
	rect.x = left;
	rect.y = top;
	rect.w = sizex;
	rect.h = sizey;
	vx = 0;
	vy = 0;
	gvx = 0;
	gvy = 0;
	__gvx = 0;
	__gvy = 0;

	if(vsizex < 0)
		vw = rect.w;
	else
		vw = vsizex;

	if(vsizey < 0)
		vh = rect.h;
	else
		vh = vsizey;
}


void window_t::select()
{
	if(!engine)
		return;

	SDL_Rect r = rect;
	selected = this;
	SDL_SetClipRect(engine->surface(), &r);
}


#if 0
void window_t::set_position(int vposx, int vposy)
{
	vx = vposx;
	vy = vposy;
	if(engine->output() != this)
	{
		gvx = vx;
		gvy = vy;
	}
}


void window_t::force_position(int vposx, int vposy)
{
	set_position(vposx, vposy);
	engine->force_scroll();
}
#endif


/*---------------------------------------------------------------
	Rendering API
---------------------------------------------------------------*/

Uint32 window_t::alloc_color(int r, int g, int b)
{
	if(!engine)
		return 0;

	return SDL_MapRGB(engine->surface()->format, r>>8, g>>8, b>>8);
}

void window_t::foreground(Uint32 color)
{
	fgcolor = color;
}

void window_t::background(Uint32 color)
{
	bgcolor = color;
}


void window_t::font(int fnt)
{
	_font = fnt;
}


void window_t::string(int x, int y, const char *txt)
{
	if(!engine)
		return;

	SoFont *f = engine->get_font(_font);
	if(!f)
		return;
	SELECT
	x += rect.x - gvx;
	y += rect.y - gvy;
	f->PutString(engine->surface(), x, y, txt);
}


void window_t::center(int y, const char *txt)
{
	if(!engine)
		return;

	SoFont *f = engine->get_font(_font);
	if(!f)
		return;
	SELECT
	int x = (rect.w - f->TextWidth(txt)) / 2;
	x += rect.x - gvx;
	y += rect.y - gvy;
	f->PutString(engine->surface(), x, y, txt);
}



void window_t::clear()
{
	if(!engine)
		return;
	SELECT
	SDL_Rect r = rect;
	SDL_FillRect(engine->surface(), &r, bgcolor);
}


void window_t::point(int x, int y)
{
	if(!engine)
		return;
	SELECT
	/* Quick hack; slow */
	SDL_Rect r;
	r.x = rect.x + x - gvx;
	r.y = rect.y + y - gvy;
	r.w = 1;
	r.h = 1;
	SDL_FillRect(engine->surface(), &r, fgcolor);
}


void window_t::fillrect(int x, int y, int w, int h)
{
	if(!engine)
		return;
	SELECT
	SDL_Rect r;
	r.x = rect.x + x - gvx;
	r.y = rect.y + y - gvy;
	r.w = w;
	r.h = h;
	SDL_FillRect(engine->surface(), &r, fgcolor);
}


void window_t::sprite(int x, int y, int bank, int frame)
{
	if(!engine)
		return;
	SELECT
	SDL_Rect dest_rect;
	SDL_Surface *img;
	img = engine->get_sprite(bank, frame);
	if(!img)
		return;

	dest_rect.x = rect.x + x - gvx;
	dest_rect.y = rect.y + y - gvy;
	SDL_BlitSurface(img, NULL, engine->surface(), &dest_rect);
}


#if 0
void window_t::get_visible_rect(SDL_Rect *r)
{
	r->x = gvx;
	r->y = gvy;
	r->w = rect.w;
	r->h = rect.h;
}


void window_t::window_to_screen(SDL_Rect *r)
{
	r->x += rect.x - gvx;
	r->y += rect.y - gvy;
}


void window_t::window_to_screen_fxp(int &x, int &y)
{
	x += PIXEL2CS(rect.x) - __gvx;
	y += PIXEL2CS(rect.y) - __gvy;
}
#endif
